/*
 * Copyright (C) 2022, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: iaom <zhangpengfei@kylinos.cn>
 *
 */
#include "file-indexer-config.h"\

#include <mutex>
#include <QDebug>
#include <QDir>
#define INDEX_SETTINGS QDir::homePath() + "/.config/org.ukui/ukui-search/ukui-search-service.conf"
/**
 * changelog: 1.1 增加ocr开关
 */
static const QString CONFIG_VERSION = QStringLiteral("1.1");
static const QByteArray UKUI_SEARCH_SCHEMAS = QByteArrayLiteral("org.ukui.search.settings");
static const QString FILE_INDEX_ENABLE_KEY = QStringLiteral("fileIndexEnable");
static const QString CONTENT_INDEX_ENABLE_KEY = QStringLiteral("contentIndexEnable");
static const QString CONTENT_FUZZY_SEARCH_KEY = QStringLiteral("contentFuzzySearch");
static const QString CONTENT_INDEX_ENABLE_OCR_KEY = QStringLiteral("contentIndexEnableOcr");
static const QString OCR_ENABLE_KEY = QStringLiteral("ocrEnable");
static const QString META_DATA_INDEX_ENABLE_KEY = QStringLiteral("metaDataIndexEnable");
static const QString CONFIG_VERSION_KEY = QStringLiteral("version");

static std::once_flag flag;
static FileIndexerConfig *global_intance = nullptr;

FileIndexerConfig *FileIndexerConfig::getInstance()
{
    std::call_once(flag, [ & ] {
        global_intance = new FileIndexerConfig();
    });
    return global_intance;
}

FileIndexerConfig::FileIndexerConfig(QObject *parent)
    : QObject(parent),
      m_dirWatcher(DirWatcher::getDirWatcher())
{
    connect(m_dirWatcher, &DirWatcher::appendIndexItem, this, &FileIndexerConfig::appendIndexDir);
    connect(m_dirWatcher, &DirWatcher::removeIndexItem, this, &FileIndexerConfig::removeIndexDir);

    const QByteArray id(UKUI_SEARCH_SCHEMAS);
    if(QGSettings::isSchemaInstalled(id)) {
        m_gsettings = new QGSettings(id, QByteArray(), this);
        //保留旧版本配置
        if(m_gsettings->keys().contains(CONFIG_VERSION_KEY)) {
            QString oldVersion = m_gsettings->get(CONFIG_VERSION_KEY).toString();
            if(oldVersion == "0.0") {
                bool fileIndexEnable = false;
                if(m_gsettings->keys().contains(FILE_INDEX_ENABLE_KEY)) {
                    fileIndexEnable = m_gsettings->get(FILE_INDEX_ENABLE_KEY).toBool();
                }
                if(fileIndexEnable) {
                    if(m_gsettings->keys().contains(CONTENT_INDEX_ENABLE_KEY)) {
                        m_gsettings->set(CONTENT_INDEX_ENABLE_KEY, true);
                    }
                    if(m_gsettings->keys().contains(CONTENT_INDEX_ENABLE_OCR_KEY)) {
                        m_gsettings->set(CONTENT_INDEX_ENABLE_OCR_KEY, true);
                    }
                }
                m_gsettings->set(CONFIG_VERSION_KEY, CONFIG_VERSION);
            } else if (oldVersion == "1.0") {
                bool contentIndex = false;
                if(m_gsettings->keys().contains(CONTENT_INDEX_ENABLE_KEY)) {
                    contentIndex = m_gsettings->get(CONTENT_INDEX_ENABLE_KEY).toBool();
                }
                if(contentIndex) {
                    if(m_gsettings->keys().contains(CONTENT_INDEX_ENABLE_OCR_KEY)) {
                        m_gsettings->set(CONTENT_INDEX_ENABLE_OCR_KEY, true);
                    }
                }
                m_gsettings->set(CONFIG_VERSION_KEY, CONFIG_VERSION);
            }
        }
        connect(m_gsettings, &QGSettings::changed, this, [ = ](const QString &key) {
            if(key == FILE_INDEX_ENABLE_KEY) {
                Q_EMIT this->fileIndexEnableStatusChanged(m_gsettings->get(FILE_INDEX_ENABLE_KEY).toBool());
            } else if(key == CONTENT_INDEX_ENABLE_KEY) {
                Q_EMIT this->contentIndexEnableStatusChanged(m_gsettings->get(CONTENT_INDEX_ENABLE_KEY).toBool());
            } else if(key == CONTENT_INDEX_ENABLE_OCR_KEY) {
                Q_EMIT this->contentIndexEnableOcrStatusChanged(m_gsettings->get(CONTENT_INDEX_ENABLE_OCR_KEY).toBool());
            }
        });
    } else {
        qWarning() << UKUI_SEARCH_SCHEMAS << " is not found!";
    }

    m_settings = new QSettings(INDEX_SETTINGS, QSettings::IniFormat, this);

}

FileIndexerConfig::~FileIndexerConfig()
{
}

QStringList FileIndexerConfig::currentIndexableDir()
{
    return DirWatcher::getDirWatcher()->currentIndexableDir();
}

QStringList FileIndexerConfig::currentBlackListOfIndex()
{
    return DirWatcher::getDirWatcher()->currentBlackListOfIndex();
}

bool FileIndexerConfig::isFileIndexEnable()
{
    if(m_gsettings) {
        if(m_gsettings->keys().contains(FILE_INDEX_ENABLE_KEY)) {
            return m_gsettings->get(FILE_INDEX_ENABLE_KEY).toBool();
        } else {
            qWarning() << "FileIndexerConfig: Can not find key:" << FILE_INDEX_ENABLE_KEY << "in" << UKUI_SEARCH_SCHEMAS;
            return false;
        }
    } else {
        qWarning() << "FileIndexerConfig:" << UKUI_SEARCH_SCHEMAS << " is not found!";
        return false;
    }
}

bool FileIndexerConfig::isContentIndexEnable()
{
    if(m_gsettings) {
        if(m_gsettings->keys().contains(CONTENT_INDEX_ENABLE_KEY)) {
            return m_gsettings->get(CONTENT_INDEX_ENABLE_KEY).toBool();
        } else {
            qWarning() << "FileIndexerConfig: Can not find key:" << CONTENT_INDEX_ENABLE_KEY << "in" << UKUI_SEARCH_SCHEMAS;
            return false;
        }
    } else {
        qWarning() << "FileIndexerConfig:" << UKUI_SEARCH_SCHEMAS << " is not found!";
        return false;
    }
}

bool FileIndexerConfig::isFuzzySearchEnable()
{
    if(m_gsettings) {
        if(m_gsettings->keys().contains(CONTENT_FUZZY_SEARCH_KEY)) {
            return m_gsettings->get(CONTENT_FUZZY_SEARCH_KEY).toBool();
        } else {
            qWarning() << "FileIndexerConfig: Can not find key:" << CONTENT_FUZZY_SEARCH_KEY << "in" << UKUI_SEARCH_SCHEMAS;
            return false;
        }
    } else {
        qWarning() << "FileIndexerConfig:" << UKUI_SEARCH_SCHEMAS << " is not found!";
        return false;
    }
}

bool FileIndexerConfig::isOCREnable()
{
    if(m_gsettings) {
        if(m_gsettings->keys().contains(CONTENT_INDEX_ENABLE_OCR_KEY)) {
            return m_gsettings->get(CONTENT_INDEX_ENABLE_OCR_KEY).toBool();
        } else {
            qWarning() << "FileIndexerConfig: Can not find key:" << CONTENT_INDEX_ENABLE_OCR_KEY << "in" << UKUI_SEARCH_SCHEMAS;
            return false;
        }
    } else {
        qWarning() << "FileIndexerConfig:" << UKUI_SEARCH_SCHEMAS << " is not found!";
        return false;
    }
}

bool FileIndexerConfig::isMetaDataIndexEnable()
{
    return m_settings->value(META_DATA_INDEX_ENABLE_KEY, true).toBool();
}
