/*
 * Copyright (C) 2022, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: iaom <zhangpengfei@kylinos.cn>
 *
 */
#include "monitor.h"
#include <QMetaEnum>
#include "file-indexer-config.h"
using namespace UkuiSearch;
Monitor::Monitor(IndexScheduler *scheduler, QObject *parent) : MonitorSource(parent),
                                                               m_scheduler(scheduler),
                                                               m_basicDatabase(DataBaseType::Basic),
                                                               m_contentDatabase(DataBaseType::Content),
                                                               m_ocrContentDatabase(DataBaseType::OcrContent)
{
    connect(scheduler, &IndexScheduler::stateChange, this, &Monitor::onIndexStateChanged);
    connect(scheduler, &IndexScheduler::process, this, &Monitor::processUpdate);
    connect(scheduler, &IndexScheduler::basicIndexDone, this, &Monitor::basicIndexDone);
    connect(scheduler, &IndexScheduler::contentIndexDone, this, &Monitor::contentIndexDone);
    connect(FileIndexerConfig::getInstance(), &FileIndexerConfig::appendIndexDir, this, [&](){
        m_currentIndexPaths = FileIndexerConfig::getInstance()->currentIndexableDir();
        Q_EMIT currentIndexPathsChanged(m_currentIndexPaths);
    });
    connect(FileIndexerConfig::getInstance(), &FileIndexerConfig::removeIndexDir, this, [&](){
        m_currentIndexPaths = FileIndexerConfig::getInstance()->currentIndexableDir();
        Q_EMIT currentIndexPathsChanged(m_currentIndexPaths);
    });
}

QStringList Monitor::currentIndexPaths() const
{
    return FileIndexerConfig::getInstance()->currentIndexableDir();
}

QString Monitor::indexState() const
{
    QMetaEnum metaEnum = QMetaEnum::fromType<IndexScheduler::IndexerState>();
    return QString::fromLocal8Bit(metaEnum.valueToKey(m_scheduler->getIndexState()));
}

uint Monitor::basicIndexSize() const
{
    return m_basicIndexSize;
}

uint Monitor::contentIndexSize() const
{
    return m_contentIndexSize;
}

uint Monitor::ocrIndexSize() const
{
    return m_ocrIndexSize;
}

uint Monitor::basicIndexProgress() const
{
    return m_basicIndexProgress;
}

uint Monitor::contentIndexProgress() const
{
    return m_contentIndexProgress;
}

uint Monitor::ocrIndexProgress() const
{
    return m_ocrIndexProgress;
}

uint Monitor::basicIndexDocNum() const
{
    return m_basicIndexDocNum;
}

uint Monitor::contentIndexDocNum() const
{
    return m_contentIndexDocNum;
}

uint Monitor::ocrContentIndexDocNum() const
{
    return m_ocrContentIndexDocNum;
}

void Monitor::onIndexStateChanged(IndexScheduler::IndexerState state)
{
    if(state == IndexScheduler::IndexerState::Idle || state == IndexScheduler::IndexerState::Stop) {
        m_basicIndexDocNum = m_basicDatabase.getIndexDocCount();
        Q_EMIT basicIndexDocNumChanged(m_basicIndexDocNum);
        m_contentIndexDocNum = m_contentDatabase.getIndexDocCount();
        Q_EMIT contentIndexDocNumChanged(m_contentIndexDocNum);
        m_ocrContentIndexDocNum = m_ocrContentDatabase.getIndexDocCount();
        Q_EMIT ocrContentIndexDocNumChanged(m_ocrContentDatabase.getIndexDocCount());
    }
    QMetaEnum metaEnum = QMetaEnum::fromType<IndexScheduler::IndexerState>();
    Q_EMIT indexStateChanged(QString::fromLocal8Bit(metaEnum.valueToKey(state)));
}

void Monitor::processUpdate(IndexType type, uint all, uint finished)
{
    switch (type) {
        case IndexType::Basic:
            m_basicIndexSize = all;
            Q_EMIT basicIndexSizeChanged(m_basicIndexSize);
            m_basicIndexProgress = finished;
            Q_EMIT basicIndexProgressChanged(m_basicIndexProgress);
            m_basicIndexDocNum = m_basicDatabase.getIndexDocCount();
            Q_EMIT basicIndexDocNumChanged(m_basicIndexDocNum);
            break;
        case IndexType::Contents:
            m_contentIndexSize = all;
            Q_EMIT contentIndexSizeChanged(m_contentIndexSize);
            m_contentIndexProgress = finished;
            Q_EMIT contentIndexProgressChanged(m_contentIndexProgress);
            m_contentIndexDocNum = m_contentDatabase.getIndexDocCount();
            Q_EMIT contentIndexDocNumChanged(m_contentIndexDocNum);
            break;
        case IndexType::OCR:
            m_ocrIndexSize = all;
            Q_EMIT ocrIndexSizeChanged(m_ocrIndexSize);
            m_ocrIndexProgress = finished;
            Q_EMIT ocrIndexProgressChanged(m_ocrIndexProgress);
            m_ocrContentIndexDocNum = m_ocrContentDatabase.getIndexDocCount();
            Q_EMIT ocrContentIndexDocNumChanged(m_ocrContentDatabase.getIndexDocCount());
            break;
        default:
            break;
    }
}
